@@ -10,16 +10,16 @@ use super::*;
 ///
 /// ```rust
 /// use tinyvec::*;
-/// 
+///
 /// let empty_av = array_vec!([u8; 16]);
-/// 
+///
 /// let some_ints = array_vec!([i32; 4], 1, 2, 3);
 /// ```
 #[macro_export]
 macro_rules! array_vec {
   ($array_type:ty) => {
     {
-      let mut av: ArrayVec<$array_type> = Default::default();
+      let av: ArrayVec<$array_type> = Default::default();
       av
     }
   };
@@ -34,9 +34,10 @@ macro_rules! array_vec {
 
 /// An array-backed vector-like data structure.
 ///
-/// * Fixed capacity (based on array size)
-/// * Variable length
-/// * All of the array memory is always initialized.
+/// * Fixed capacity (based on array size).
+/// * Variable length.
+/// * All of the array memory is always "initialized" in the init/uninit memory
+///   sense.
 #[repr(C)]
 #[derive(Clone, Copy, Default)]
 pub struct ArrayVec<A: Array> {
@@ -49,7 +50,7 @@ impl<A: Array> Deref for ArrayVec<A> {
   #[inline(always)]
   #[must_use]
   fn deref(&self) -> &Self::Target {
-    &self.data.slice()[..self.len]
+    &self.data.as_slice()[..self.len]
   }
 }
 
@@ -57,7 +58,7 @@ impl<A: Array> DerefMut for ArrayVec<A> {
   #[inline(always)]
   #[must_use]
   fn deref_mut(&mut self) -> &mut Self::Target {
-    &mut self.data.slice_mut()[..self.len]
+    &mut self.data.as_slice_mut()[..self.len]
   }
 }
 
@@ -80,6 +81,19 @@ impl<A: Array, I: SliceIndex<[A::Item]>> IndexMut<I> for ArrayVec<A> {
 
 impl<A: Array> ArrayVec<A> {
   /// Move all values from `other` into this vec.
+  ///
+  /// ## Panics
+  /// * If the vec overflows its capacity
+  ///
+  /// ## Example
+  /// ```rust
+  /// use tinyvec::*;
+  /// let mut av = array_vec!([i32; 10], 1, 2, 3);
+  /// let mut av2 = array_vec!([i32; 10], 4, 5, 6);
+  /// av.append(&mut av2);
+  /// assert_eq!(av, &[1, 2, 3, 4, 5, 6][..]);
+  /// assert_eq!(av2, &[][..]);
+  /// ```
   #[inline]
   pub fn append(&mut self, other: &mut Self) {
     for item in other.drain(..) {
@@ -88,14 +102,14 @@ impl<A: Array> ArrayVec<A> {
   }
 
   /// A mutable pointer to the backing array.
-  /// 
+  ///
   /// ## Safety
-  /// 
+  ///
   /// This pointer has provenance over the _entire_ backing array.
   #[inline(always)]
   #[must_use]
   pub fn as_mut_ptr(&mut self) -> *mut A::Item {
-    self.data.slice_mut().as_mut_ptr()
+    self.data.as_slice_mut().as_mut_ptr()
   }
 
   /// Helper for getting the mut slice.
@@ -106,14 +120,14 @@ impl<A: Array> ArrayVec<A> {
   }
 
   /// A const pointer to the backing array.
-  /// 
+  ///
   /// ## Safety
-  /// 
+  ///
   /// This pointer has provenance over the _entire_ backing array.
   #[inline(always)]
   #[must_use]
   pub fn as_ptr(&self) -> *const A::Item {
-    self.data.slice().as_ptr()
+    self.data.as_slice().as_ptr()
   }
 
   /// Helper for getting the shared slice.
@@ -124,7 +138,7 @@ impl<A: Array> ArrayVec<A> {
   }
 
   /// The capacity of the `ArrayVec`.
-  /// 
+  ///
   /// This is fixed based on the array type.
   #[inline(always)]
   #[must_use]
@@ -234,9 +248,22 @@ impl<A: Array> ArrayVec<A> {
   where
     A::Item: Clone,
   {
-    for i in sli {
-      self.push(i.clone())
+    if sli.is_empty() {
+      return;
     }
+
+    let new_len = self.len + sli.len();
+    if new_len > A::CAPACITY {
+      panic!(
+        "ArrayVec::extend_from_slice: total length {} exceeds capacity {}!",
+        new_len,
+        A::CAPACITY
+      )
+    }
+
+    let target = &mut self.data.as_slice_mut()[self.len..new_len];
+    target.clone_from_slice(sli);
+    self.set_len(new_len);
   }
 
   /// Wraps up an array and uses the given length as the initial length.
@@ -262,7 +289,8 @@ impl<A: Array> ArrayVec<A> {
   /// index.
   ///
   /// ## Panics
-  /// * If `index` > `len`
+  /// * If `index` > `len` or
+  /// * If the capacity is exhausted
   ///
   /// ## Example
   /// ```rust
@@ -275,26 +303,14 @@ impl<A: Array> ArrayVec<A> {
   /// ```
   #[inline]
   pub fn insert(&mut self, index: usize, item: A::Item) {
-    use core::cmp::Ordering;
-    match index.cmp(&self.len) {
-      Ordering::Less => {
-        let targets: &mut [A::Item] = &mut self.as_mut_slice()[index..];
-        let mut temp = item;
-        for target in targets.iter_mut() {
-          temp = replace(target, temp);
-        }
-        self.push(temp);
-      }
-      Ordering::Equal => {
-        self.push(item);
-      }
-      Ordering::Greater => {
-        panic!(
-          "ArrayVec::insert> index {} is out of bounds {}",
-          index, self.len
-        );
-      }
+    if index > self.len {
+      panic!("ArrayVec::insert> index {} is out of bounds {}", index, self.len);
     }
+
+    // Try to push the element.
+    self.push(item);
+    // And move it into its place.
+    self.as_mut_slice()[index..].rotate_right(1);
   }
 
   /// If the vec is empty.
@@ -322,15 +338,14 @@ impl<A: Array> ArrayVec<A> {
   }
 
   /// Remove and return the last element of the vec, if there is one.
-  /// 
+  ///
   /// ## Failure
   /// * If the vec is empty you get `None`.
   #[inline]
   pub fn pop(&mut self) -> Option<A::Item> {
     if self.len > 0 {
       self.len -= 1;
-      let out =
-        replace(&mut self.data.slice_mut()[self.len], A::Item::default());
+      let out = take(&mut self.data.as_slice_mut()[self.len]);
       Some(out)
     } else {
       None
@@ -338,14 +353,16 @@ impl<A: Array> ArrayVec<A> {
   }
 
   /// Place an element onto the end of the vec.
-  /// 
-  /// See also, [`try_push`](ArrayVec::try_push)
+  ///
   /// ## Panics
   /// * If the length of the vec would overflow the capacity.
   #[inline(always)]
   pub fn push(&mut self, val: A::Item) {
-    if self.try_push(val).is_err() {
-      panic!("ArrayVec: overflow!")
+    if self.len < A::CAPACITY {
+      replace(&mut self.data.as_slice_mut()[self.len], val);
+      self.len += 1;
+    } else {
+      panic!("ArrayVec::push> capacity overflow!")
     }
   }
 
@@ -368,12 +385,10 @@ impl<A: Array> ArrayVec<A> {
   #[inline]
   pub fn remove(&mut self, index: usize) -> A::Item {
     let targets: &mut [A::Item] = &mut self.deref_mut()[index..];
-    let mut spare = A::Item::default();
-    for target in targets.iter_mut().rev() {
-      spare = replace(target, spare);
-    }
+    let item = replace(&mut targets[0], A::Item::default());
+    targets.rotate_left(1);
     self.len -= 1;
-    spare
+    item
   }
 
   // NIGHTLY: remove_item, https://github.com/rust-lang/rust/issues/40062
@@ -401,14 +416,14 @@ impl<A: Array> ArrayVec<A> {
   where
     A::Item: Clone,
   {
-    use core::cmp::Ordering;
-    match new_len.cmp(&self.len) {
-      Ordering::Less => self.truncate(new_len),
-      Ordering::Equal => (),
-      Ordering::Greater => {
-        while self.len < new_len {
+    match new_len.checked_sub(self.len) {
+      None => self.truncate(new_len),
+      Some(0) => (),
+      Some(new_elements) => {
+        for _ in 1..new_elements {
           self.push(new_val.clone());
         }
+        self.push(new_val);
       }
     }
   }
@@ -441,12 +456,10 @@ impl<A: Array> ArrayVec<A> {
     new_len: usize,
     mut f: F,
   ) {
-    use core::cmp::Ordering;
-    match new_len.cmp(&self.len) {
-      Ordering::Less => self.truncate(new_len),
-      Ordering::Equal => (),
-      Ordering::Greater => {
-        while self.len < new_len {
+    match new_len.checked_sub(self.len) {
+      None => self.truncate(new_len),
+      Some(new_elements) => {
+        for _ in 0..new_elements {
           self.push(f());
         }
       }
@@ -460,22 +473,105 @@ impl<A: Array> ArrayVec<A> {
   /// ```rust
   /// use tinyvec::*;
   ///
-  /// let mut av = array_vec!([i32; 10], 1, 2, 3, 4);
+  /// let mut av = array_vec!([i32; 10], 1, 1, 2, 3, 3, 4);
   /// av.retain(|&x| x % 2 == 0);
   /// assert_eq!(av.as_slice(), &[2, 4][..]);
   /// ```
   #[inline]
   pub fn retain<F: FnMut(&A::Item) -> bool>(&mut self, mut acceptable: F) {
-    let mut i = 0;
-    while i < self.len {
-      if !acceptable(&self[i]) {
-        self.remove(i);
+    // Drop guard to contain exactly the remaining elements when the test
+    // panics.
+    struct JoinOnDrop<'vec, Item> {
+      items: &'vec mut [Item],
+      done_end: usize,
+      // Start of tail relative to `done_end`.
+      tail_start: usize,
+    }
+
+    impl<Item> Drop for JoinOnDrop<'_, Item> {
+      fn drop(&mut self) {
+        self.items[self.done_end..].rotate_left(self.tail_start);
+      }
+    }
+
+    let mut rest = JoinOnDrop {
+      items: &mut self.data.as_slice_mut()[..self.len],
+      done_end: 0,
+      tail_start: 0,
+    };
+
+    for idx in 0..self.len {
+      // Loop start invariant: idx = rest.done_end + rest.tail_start
+      if !acceptable(&rest.items[idx]) {
+        let _ = take(&mut rest.items[idx]);
+        self.len -= 1;
+        rest.tail_start += 1;
+      } else {
+        rest.items.swap(rest.done_end, idx);
+        rest.done_end += 1;
       }
-      i += 1;
     }
   }
 
-  // LATER(Vec): splice
+  /// Forces the length of the vector to `new_len`.
+  ///
+  /// ## Panics
+  /// If `new_len` is greater than the vec's capacity.
+  ///
+  /// ## Safety
+  /// * This is a fully safe operation! The inactive memory already counts as
+  ///   "initialized" by Rust's rules.
+  /// * Other than "the memory is initialized" there are no other guarantees
+  ///   regarding what you find in the inactive portion of the vec.
+  #[inline(always)]
+  pub fn set_len(&mut self, new_len: usize) {
+    if new_len > A::CAPACITY {
+      // Note(Lokathor): Technically we don't have to panic here, and we could
+      // just let some other call later on trigger a panic on accident when the
+      // length is wrong. However, it's a lot easier to catch bugs when things
+      // are more "fail-fast".
+      panic!("ArrayVec: set_len overflow!")
+    } else {
+      self.len = new_len;
+    }
+  }
+
+  /// Fill the vector until its capacity has been reached.
+  ///
+  /// Successively fills unused space in the spare slice of the vector with
+  /// elements from the iterator. It then returns the remaining iterator
+  /// without exhausting it. This also allows appending the head of an
+  /// infinite iterator.
+  ///
+  /// This is an alternative to `Extend::extend` method for cases where the
+  /// length of the iterator can not be checked. Since this vector can not
+  /// reallocate to increase its capacity, it is unclear what to do with
+  /// remaining elements in the iterator and the iterator itself. The
+  /// interface also provides no way to communicate this to the caller.
+  ///
+  /// ## Example
+  ///
+  /// ```rust
+  /// use tinyvec::*;
+  /// let mut av = array_vec!([i32; 4]);
+  /// let mut to_inf = av.fill(0..);
+  /// assert_eq!(av.as_slice(), &[0, 1, 2, 3][..]);
+  /// assert_eq!(to_inf.next(), Some(4));
+  /// ```
+  ///
+  /// ## Panics
+  /// Only panics if the `next` method of the provided iterator panics.
+  #[inline]
+  pub fn fill<I: IntoIterator<Item = A::Item>>(
+    &mut self,
+    iter: I,
+  ) -> I::IntoIter {
+    let mut iter = iter.into_iter();
+    for element in iter.by_ref().take(self.capacity() - self.len()) {
+      self.push(element);
+    }
+    iter
+  }
 
   /// Splits the collection at the point given.
   ///
@@ -508,11 +604,10 @@ impl<A: Array> ArrayVec<A> {
     }
     let mut new = Self::default();
     let moves = &mut self.as_mut_slice()[at..];
-    let targets = new.data.slice_mut();
-    for (m, t) in moves.iter_mut().zip(targets) {
-      replace(t, replace(m, A::Item::default()));
-    }
-    new.len = self.len - at;
+    let split_len = moves.len();
+    let targets = &mut new.data.as_slice_mut()[..split_len];
+    moves.swap_with_slice(targets);
+    new.len = split_len;
     self.len = at;
     new
   }
@@ -542,15 +637,15 @@ impl<A: Array> ArrayVec<A> {
       self.len
     );
     if index == self.len - 1 {
-        self.pop().unwrap()
+      self.pop().unwrap()
     } else {
-        let i = self.pop().unwrap();
-        replace(&mut self[index], i)
+      let i = self.pop().unwrap();
+      replace(&mut self[index], i)
     }
   }
 
   /// Reduces the vec's length to the given value.
-  /// 
+  ///
   /// If the vec is already shorter than the input, nothing happens.
   #[inline]
   pub fn truncate(&mut self, new_len: usize) {
@@ -581,30 +676,45 @@ impl<A: Array> ArrayVec<A> {
     }
   }
 
-  /// Pushes an item if there's room.
+  /// Obtain the shared slice of the array _after_ the active memory.
   ///
-  /// ## Failure
-  ///
-  /// If there's no more capacity the vec is unchanged, and you get the item
-  /// back in the `Err`.
-  #[inline]
-  pub fn try_push(&mut self, val: A::Item) -> Result<(), A::Item> {
-    if self.len < A::CAPACITY {
-      replace(&mut self.data.slice_mut()[self.len], val);
-      self.len += 1;
-      Ok(())
-    } else {
-      Err(val)
-    }
+  /// ## Example
+  /// ```rust
+  /// use tinyvec::*;
+  /// let mut av = array_vec!([i32; 4]);
+  /// assert_eq!(av.grab_spare_slice().len(), 4);
+  /// av.push(10);
+  /// av.push(11);
+  /// av.push(12);
+  /// av.push(13);
+  /// assert_eq!(av.grab_spare_slice().len(), 0);
+  /// ```
+  #[inline(always)]
+  #[cfg(feature = "grab_spare_slice")]
+  pub fn grab_spare_slice(&self) -> &[A::Item] {
+    &self.data.as_slice()[self.len..]
   }
 
-  // LATER: try_insert ?
-
-  // LATER: try_remove ?
+  /// Obtain the mutable slice of the array _after_ the active memory.
+  ///
+  /// ## Example
+  /// ```rust
+  /// use tinyvec::*;
+  /// let mut av = array_vec!([i32; 4]);
+  /// assert_eq!(av.grab_spare_slice_mut().len(), 4);
+  /// av.push(10);
+  /// av.push(11);
+  /// assert_eq!(av.grab_spare_slice_mut().len(), 2);
+  /// ```
+  #[inline(always)]
+  #[cfg(feature = "grab_spare_slice")]
+  pub fn grab_spare_slice_mut(&mut self) -> &mut [A::Item] {
+    &mut self.data.as_slice_mut()[self.len..]
+  }
 }
 
 /// Draining iterator for `ArrayVecDrain`
-/// 
+///
 /// See [`ArrayVecDrain::drain`](ArrayVecDrain::<A>::drain)
 pub struct ArrayVecDrain<'p, A: Array> {
   parent: &'p mut ArrayVec<A>,
@@ -682,7 +792,7 @@ impl<A: Array> From<A> for ArrayVec<A> {
   /// If you want to select a length, use
   /// [`from_array_len`](ArrayVec::from_array_len)
   fn from(data: A) -> Self {
-    Self { len: data.slice().len(), data }
+    Self { len: data.as_slice().len(), data }
   }
 }
 
@@ -709,8 +819,7 @@ impl<A: Array> Iterator for ArrayVecIterator<A> {
   #[inline]
   fn next(&mut self) -> Option<Self::Item> {
     if self.base < self.len {
-      let out =
-        replace(&mut self.data.slice_mut()[self.base], A::Item::default());
+      let out = take(&mut self.data.as_slice_mut()[self.base]);
       self.base += 1;
       Some(out)
     } else {
@@ -729,13 +838,13 @@ impl<A: Array> Iterator for ArrayVecIterator<A> {
   }
   #[inline]
   fn last(mut self) -> Option<Self::Item> {
-    Some(replace(&mut self.data.slice_mut()[self.len], A::Item::default()))
+    Some(take(&mut self.data.as_slice_mut()[self.len]))
   }
   #[inline]
   fn nth(&mut self, n: usize) -> Option<A::Item> {
     let i = self.base + (n - 1);
     if i < self.len {
-      let out = replace(&mut self.data.slice_mut()[i], A::Item::default());
+      let out = take(&mut self.data.as_slice_mut()[i]);
       self.base = i + 1;
       Some(out)
     } else {
@@ -794,7 +903,7 @@ where
   #[inline]
   #[must_use]
   fn eq(&self, other: &&A) -> bool {
-    self.deref() == other.slice()
+    self.deref() == other.as_slice()
   }
 }
 
