@@ -14,9 +14,9 @@ use alloc::vec::Vec;
 ///
 /// ```rust
 /// use tinyvec::*;
-/// 
+///
 /// let empty_av = tiny_vec!([u8; 16]);
-/// 
+///
 /// let some_ints = tiny_vec!([i32; 4], 1, 2, 3);
 /// ```
 #[macro_export]
@@ -51,7 +51,7 @@ pub enum TinyVec<A: Array> {
   #[allow(missing_docs)]
   Inline(ArrayVec<A>),
   #[allow(missing_docs)]
-  Heap(Vec<A::Item>)
+  Heap(Vec<A::Item>),
 }
 impl<A: Array + Default> Default for TinyVec<A> {
   #[inline]
@@ -127,9 +127,9 @@ impl<A: Array> TinyVec<A> {
   }
 
   /// A mutable pointer to the backing array.
-  /// 
+  ///
   /// ## Safety
-  /// 
+  ///
   /// This pointer has provenance over the _entire_ backing array/buffer.
   #[inline(always)]
   #[must_use]
@@ -148,9 +148,9 @@ impl<A: Array> TinyVec<A> {
   }
 
   /// A const pointer to the backing array.
-  /// 
+  ///
   /// ## Safety
-  /// 
+  ///
   /// This pointer has provenance over the _entire_ backing array/buffer.
   #[inline(always)]
   #[must_use]
@@ -169,12 +169,16 @@ impl<A: Array> TinyVec<A> {
   }
 
   /// The capacity of the `TinyVec`.
-  /// 
-  /// This is fixed based on the array type.
+  ///
+  /// When not heap allocated this is fixed based on the array type.
+  /// Otherwise its the result of the underlying Vec::capacity.
   #[inline(always)]
   #[must_use]
   pub fn capacity(&self) -> usize {
-    A::CAPACITY
+    match self {
+      TinyVec::Inline(_) => A::CAPACITY,
+      TinyVec::Heap(v) => v.capacity(),
+    }
   }
 
   /// Removes all elements from the vec.
@@ -182,7 +186,7 @@ impl<A: Array> TinyVec<A> {
   pub fn clear(&mut self) {
     self.truncate(0)
   }
-  
+
   /// De-duplicates the vec.
   #[cfg(feature = "nightly_slice_partition_dedup")]
   #[inline(always)]
@@ -321,12 +325,14 @@ impl<A: Array> TinyVec<A> {
   #[inline]
   pub fn insert(&mut self, index: usize, item: A::Item) {
     match self {
-      TinyVec::Inline(a) => if a.len() == A::CAPACITY {
-        self.move_to_the_heap();
-        self.insert(index, item)
-      } else {
-        a.insert(index, item);
-      },
+      TinyVec::Inline(a) => {
+        if a.len() == A::CAPACITY {
+          self.move_to_the_heap();
+          self.insert(index, item)
+        } else {
+          a.insert(index, item);
+        }
+      }
       TinyVec::Heap(v) => v.insert(index, item),
     }
   }
@@ -359,7 +365,7 @@ impl<A: Array> TinyVec<A> {
   }
 
   /// Remove and return the last element of the vec, if there is one.
-  /// 
+  ///
   /// ## Failure
   /// * If the vec is empty you get `None`.
   #[inline]
@@ -371,19 +377,19 @@ impl<A: Array> TinyVec<A> {
   }
 
   /// Place an element onto the end of the vec.
-  /// 
-  /// See also, [`try_push`](TinyVec::<A>::try_push)
   /// ## Panics
   /// * If the length of the vec would overflow the capacity.
   #[inline(always)]
   pub fn push(&mut self, val: A::Item) {
     match self {
-      TinyVec::Inline(a) => if a.len() == A::CAPACITY {
-        self.move_to_the_heap();
-        self.push(val)
-      } else {
-        a.push(val);
-      },
+      TinyVec::Inline(a) => {
+        if a.len() == A::CAPACITY {
+          self.move_to_the_heap();
+          self.push(val)
+        } else {
+          a.push(val);
+        }
+      }
       TinyVec::Heap(v) => v.push(val),
     }
   }
@@ -438,12 +444,14 @@ impl<A: Array> TinyVec<A> {
     A::Item: Clone,
   {
     match self {
-      TinyVec::Inline(a) => if new_len > A::CAPACITY {
-        self.move_to_the_heap();
-        self.resize(new_len, new_val);
-      } else {
-        a.resize(new_len, new_val);
-      },
+      TinyVec::Inline(a) => {
+        if new_len > A::CAPACITY {
+          self.move_to_the_heap();
+          self.resize(new_len, new_val);
+        } else {
+          a.resize(new_len, new_val);
+        }
+      }
       TinyVec::Heap(v) => v.resize(new_len, new_val),
     }
   }
@@ -471,11 +479,7 @@ impl<A: Array> TinyVec<A> {
   /// assert_eq!(tv.as_slice(), &[2, 4, 8, 16][..]);
   /// ```
   #[inline]
-  pub fn resize_with<F: FnMut() -> A::Item>(
-    &mut self,
-    new_len: usize,
-    f: F,
-  ) {
+  pub fn resize_with<F: FnMut() -> A::Item>(&mut self, new_len: usize, f: F) {
     match self {
       TinyVec::Inline(a) => a.resize_with(new_len, f),
       TinyVec::Heap(v) => v.resize_with(new_len, f),
@@ -556,7 +560,7 @@ impl<A: Array> TinyVec<A> {
   }
 
   /// Reduces the vec's length to the given value.
-  /// 
+  ///
   /// If the vec is already shorter than the input, nothing happens.
   #[inline]
   pub fn truncate(&mut self, new_len: usize) {
@@ -580,16 +584,10 @@ impl<A: Array> TinyVec<A> {
     let arr = ArrayVec::try_from_array_len(data, len)?;
     Ok(TinyVec::Inline(arr))
   }
-
-  // LATER: try_push ?
-
-  // LATER: try_insert ?
-
-  // LATER: try_remove ?
 }
 
 /// Draining iterator for `TinyVecDrain`
-/// 
+///
 /// See [`TinyVecDrain::drain`](TinyVecDrain::<A>::drain)
 pub struct TinyVecDrain<'p, A: Array> {
   parent: &'p mut TinyVec<A>,
@@ -684,7 +682,7 @@ pub enum TinyVecIterator<A: Array> {
   #[allow(missing_docs)]
   Inline(ArrayVecIterator<A>),
   #[allow(missing_docs)]
-  Heap(alloc::vec::IntoIter<A::Item>)
+  Heap(alloc::vec::IntoIter<A::Item>),
 }
 impl<A: Array> Iterator for TinyVecIterator<A> {
   type Item = A::Item;
@@ -779,7 +777,7 @@ where
   #[inline]
   #[must_use]
   fn eq(&self, other: &&A) -> bool {
-    self.deref() == other.slice()
+    self.as_slice() == other.as_slice()
   }
 }
 
